#include "base/exception.hpp"
#include "base/objectlock.hpp"
#include "base/utility.hpp"
#include "base/convert.hpp"
#include "base/dependencygraph.hpp"
#include "base/logger.hpp"
#include "base/function.hpp"
#include "base/configtype.hpp"
#ifdef _MSC_VER
#pragma warning( push )
#pragma warning( disable : 4244 )
#pragma warning( disable : 4800 )
#endif /* _MSC_VER */

namespace icinga
{

TypeImpl<StatusDataWriter>::TypeImpl()
{ }

TypeImpl<StatusDataWriter>::~TypeImpl()
{ }

String TypeImpl<StatusDataWriter>::GetName() const
{
	return "StatusDataWriter";
}

int TypeImpl<StatusDataWriter>::GetAttributes() const
{
	return 0;
}

Type::Ptr TypeImpl<StatusDataWriter>::GetBaseType() const
{
	return ConfigObject::TypeInstance;
}

int TypeImpl<StatusDataWriter>::GetFieldId(const String& name) const
{
	int offset = ConfigObject::TypeInstance->GetFieldCount();

	switch (static_cast<int>(Utility::SDBM(name, 1))) {
		case 111:
			if (name == "objects_path")
				return offset + 1;

			break;
		case 115:
			if (name == "status_path")
				return offset + 0;

			break;
		case 117:
			if (name == "update_interval")
				return offset + 2;

			break;
	}

	return ConfigObject::TypeInstance->GetFieldId(name);
}

Field TypeImpl<StatusDataWriter>::GetFieldInfo(int id) const
{
	int real_id = id - ConfigObject::TypeInstance->GetFieldCount();
	if (real_id < 0) { return ConfigObject::TypeInstance->GetFieldInfo(id); }
	switch (real_id) {
		case 0:
			return {0, "String", "status_path", "status_path", nullptr, 2, 0};
		case 1:
			return {1, "String", "objects_path", "objects_path", nullptr, 2, 0};
		case 2:
			return {2, "Number", "update_interval", "update_interval", nullptr, 2, 0};
		default:
			throw std::runtime_error("Invalid field ID.");
	}
}

int TypeImpl<StatusDataWriter>::GetFieldCount() const
{
	return 3 + ConfigObject::TypeInstance->GetFieldCount();
}

ObjectFactory TypeImpl<StatusDataWriter>::GetFactory() const
{
	return TypeHelper<StatusDataWriter, false>::GetFactory();
}

std::vector<String> TypeImpl<StatusDataWriter>::GetLoadDependencies() const
{
	std::vector<String> deps;
	return deps;
}

int TypeImpl<StatusDataWriter>::GetActivationPriority() const
{
	return 100;
}

void TypeImpl<StatusDataWriter>::RegisterAttributeHandler(int fieldId, const Type::AttributeHandler& callback)
{
	int real_id = fieldId - ConfigObject::TypeInstance->GetFieldCount(); 
	if (real_id < 0) { ConfigObject::TypeInstance->RegisterAttributeHandler(fieldId, callback); return; }
	switch (real_id) {
		case 0:
			ObjectImpl<StatusDataWriter>::OnStatusPathChanged.connect(callback);
			break;
		case 1:
			ObjectImpl<StatusDataWriter>::OnObjectsPathChanged.connect(callback);
			break;
		case 2:
			ObjectImpl<StatusDataWriter>::OnUpdateIntervalChanged.connect(callback);
			break;
		default:
			throw std::runtime_error("Invalid field ID.");
	}
}

void ObjectImpl<StatusDataWriter>::Validate(int types, const ValidationUtils& utils)
{
	ConfigObject::Validate(types, utils);

	if (2 & types)
		ValidateStatusPath(Lazy<String>([this]() { return GetStatusPath(); }), utils);
	if (2 & types)
		ValidateObjectsPath(Lazy<String>([this]() { return GetObjectsPath(); }), utils);
	if (2 & types)
		ValidateUpdateInterval(Lazy<double>([this]() { return GetUpdateInterval(); }), utils);
}

void ObjectImpl<StatusDataWriter>::SimpleValidateStatusPath(const Lazy<String>& value, const ValidationUtils& utils)
{
}

void ObjectImpl<StatusDataWriter>::SimpleValidateObjectsPath(const Lazy<String>& value, const ValidationUtils& utils)
{
}

void ObjectImpl<StatusDataWriter>::SimpleValidateUpdateInterval(const Lazy<double>& value, const ValidationUtils& utils)
{
}

ObjectImpl<StatusDataWriter>::ObjectImpl()
{
	SetStatusPath(GetDefaultStatusPath(), true);
	SetObjectsPath(GetDefaultObjectsPath(), true);
	SetUpdateInterval(GetDefaultUpdateInterval(), true);
}

ObjectImpl<StatusDataWriter>::~ObjectImpl()
{ }

void ObjectImpl<StatusDataWriter>::SetField(int id, const Value& value, bool suppress_events, const Value& cookie)
{
	int real_id = id - ConfigObject::TypeInstance->GetFieldCount(); 
	if (real_id < 0) { ConfigObject::SetField(id, value, suppress_events, cookie); return; }
	switch (real_id) {
		case 0:
			SetStatusPath(value, suppress_events, cookie);
			break;
		case 1:
			SetObjectsPath(value, suppress_events, cookie);
			break;
		case 2:
			SetUpdateInterval(value, suppress_events, cookie);
			break;
		default:
			throw std::runtime_error("Invalid field ID.");
	}
}

Value ObjectImpl<StatusDataWriter>::GetField(int id) const
{
	int real_id = id - ConfigObject::TypeInstance->GetFieldCount(); 
	if (real_id < 0) { return ConfigObject::GetField(id); }
	switch (real_id) {
		case 0:
			return GetStatusPath();
		case 1:
			return GetObjectsPath();
		case 2:
			return GetUpdateInterval();
		default:
			throw std::runtime_error("Invalid field ID.");
	}
}

void ObjectImpl<StatusDataWriter>::ValidateField(int id, const Lazy<Value>& lvalue, const ValidationUtils& utils)
{
	int real_id = id - ConfigObject::TypeInstance->GetFieldCount(); 
	if (real_id < 0) { ConfigObject::ValidateField(id, lvalue, utils); return; }
	switch (real_id) {
		case 0:
			ValidateStatusPath(lvalue, utils);
			break;
		case 1:
			ValidateObjectsPath(lvalue, utils);
			break;
		case 2:
			ValidateUpdateInterval(lvalue, utils);
			break;
		default:
			throw std::runtime_error("Invalid field ID.");
	}
}

void ObjectImpl<StatusDataWriter>::NotifyField(int id, const Value& cookie)
{
	int real_id = id - ConfigObject::TypeInstance->GetFieldCount(); 
	if (real_id < 0) { ConfigObject::NotifyField(id, cookie); return; }
	switch (real_id) {
		case 0:
			NotifyStatusPath(cookie);
			break;
		case 1:
			NotifyObjectsPath(cookie);
			break;
		case 2:
			NotifyUpdateInterval(cookie);
			break;
		default:
			throw std::runtime_error("Invalid field ID.");
	}
}

Object::Ptr ObjectImpl<StatusDataWriter>::NavigateField(int id) const
{
	int real_id = id - ConfigObject::TypeInstance->GetFieldCount(); 
	if (real_id < 0) { return ConfigObject::NavigateField(id); }
	throw std::runtime_error("Invalid field ID.");
}

String ObjectImpl<StatusDataWriter>::GetStatusPath() const
{
	return m_StatusPath.load();
}

String ObjectImpl<StatusDataWriter>::GetObjectsPath() const
{
	return m_ObjectsPath.load();
}

double ObjectImpl<StatusDataWriter>::GetUpdateInterval() const
{
	return m_UpdateInterval.load();
}

void ObjectImpl<StatusDataWriter>::SetStatusPath(const String& value, bool suppress_events, const Value& cookie)
{
	m_StatusPath.store(value);
	if (!suppress_events)
		NotifyStatusPath(cookie);
}

void ObjectImpl<StatusDataWriter>::SetObjectsPath(const String& value, bool suppress_events, const Value& cookie)
{
	m_ObjectsPath.store(value);
	if (!suppress_events)
		NotifyObjectsPath(cookie);
}

void ObjectImpl<StatusDataWriter>::SetUpdateInterval(double value, bool suppress_events, const Value& cookie)
{
	m_UpdateInterval.store(value);
	if (!suppress_events)
		NotifyUpdateInterval(cookie);
}

void ObjectImpl<StatusDataWriter>::NotifyStatusPath(const Value& cookie)
{
	auto *dobj = dynamic_cast<ConfigObject *>(this);
	if (!dobj || dobj->IsActive())
		OnStatusPathChanged(static_cast<StatusDataWriter *>(this), cookie);
}

void ObjectImpl<StatusDataWriter>::NotifyObjectsPath(const Value& cookie)
{
	auto *dobj = dynamic_cast<ConfigObject *>(this);
	if (!dobj || dobj->IsActive())
		OnObjectsPathChanged(static_cast<StatusDataWriter *>(this), cookie);
}

void ObjectImpl<StatusDataWriter>::NotifyUpdateInterval(const Value& cookie)
{
	auto *dobj = dynamic_cast<ConfigObject *>(this);
	if (!dobj || dobj->IsActive())
		OnUpdateIntervalChanged(static_cast<StatusDataWriter *>(this), cookie);
}

String ObjectImpl<StatusDataWriter>::GetDefaultStatusPath() const
{
	 return Configuration::CacheDir + "/status.dat"; 
}

String ObjectImpl<StatusDataWriter>::GetDefaultObjectsPath() const
{
	 return Configuration::CacheDir + "/objects.cache"; 
}

double ObjectImpl<StatusDataWriter>::GetDefaultUpdateInterval() const
{
	 return 15; 
}


boost::signals2::signal<void (const intrusive_ptr<StatusDataWriter>&, const Value&)> ObjectImpl<StatusDataWriter>::OnStatusPathChanged;


boost::signals2::signal<void (const intrusive_ptr<StatusDataWriter>&, const Value&)> ObjectImpl<StatusDataWriter>::OnObjectsPathChanged;


boost::signals2::signal<void (const intrusive_ptr<StatusDataWriter>&, const Value&)> ObjectImpl<StatusDataWriter>::OnUpdateIntervalChanged;

void ObjectImpl<StatusDataWriter>::ValidateObjectsPath(const Lazy<String>& lvalue, const ValidationUtils& utils)
{
	SimpleValidateObjectsPath(lvalue, utils);
}

void ObjectImpl<StatusDataWriter>::ValidateStatusPath(const Lazy<String>& lvalue, const ValidationUtils& utils)
{
	SimpleValidateStatusPath(lvalue, utils);
}

void ObjectImpl<StatusDataWriter>::ValidateUpdateInterval(const Lazy<double>& lvalue, const ValidationUtils& utils)
{
	SimpleValidateUpdateInterval(lvalue, utils);
}

}
#ifdef _MSC_VER
#pragma warning ( pop )
#endif /* _MSC_VER */
